@@ -4,21 +4,20 @@ package Statistics::Gtest;
 
 use strict;
 use vars qw($VERSION);
+use Carp;
 use IO::File;
 
-$VERSION = '0.04';
+$VERSION = '0.05';
 
 my $self;
 
 sub new {
-   my ($type, $data, $overflow) = @_;
+   my ($type, $data) = @_;
    my $datahandle;
    if (! $data) {
       _error(1);
    }
-	# The only purpose of this variable is to detect if too many parameters
-	# have been passed to the constructor.
-   if ($overflow) {
+   if (@_ eq 3) {
       _error(2);
    }      
    if ($datahandle = _getHandle($data)) {
@@ -52,13 +51,11 @@ sub getQ {
 }
 
 sub setExpected {
-   my ($self, $expData, $overflow) = @_;
+   my ($self, $expData) = @_;
    my $datahandle;
    my @expect;
    my $expectTotal = 0;
-	# The only purpose of this variable is to detect if too many parameters
-	# have been passed to the constructor.
-   if ($overflow) {
+   if (@_ eq 3) {
       _error(2);
    }      
    if ($datahandle = _getHandle($expData)) {
@@ -84,12 +81,24 @@ sub setExpected {
    # Data sanity checks
    if ($expectTotal != $self->{'sumtotal'}) {
       warn "Warning: Total of expected values ($expectTotal) does not ",
-		"equal total of observed values (",$self->{'sumtotal'},").\nThis ",
-		"will invalidate the test unless the discrepancy is very minor ",
-		"\n(e.g., the result of rounding error).\n";
+      "equal total of observed values (",$self->{'sumtotal'},").\nThis ",
+      "will invalidate the test unless the discrepancy is very minor ",
+      "\n(e.g., the result of rounding error).\n";
    }
 }
 
+sub getObserved {
+   my $self = shift;
+   my $exp = _formatData($self->{'observed'});
+   $exp;
+}
+
+sub getExpected {
+   my $self = shift;
+   my $exp = _formatData($self->{'expected'});
+   $exp;
+}
+
 sub getDF {
    my $self = shift;
    $self->{'df'};
@@ -102,6 +111,16 @@ sub setDF {
    $self->{'df_extern'} = 1;
 }
 
+sub getDFstate {
+   my $self = shift;
+   $self->{'df_extern'};
+}
+
+sub getHypothesis {
+   my $self = shift;
+   $self->{'intrinsic'};
+}
+
 sub getRow {
    my $self = shift;
    my $idx = shift;
@@ -325,6 +344,22 @@ sub _rowsum {
    $total;
 }
 
+# Order the data array back into the format originally input.
+sub _formatData {
+   my $ref = shift;
+   my @data;
+   my $len =  scalar @{$ref};
+   my $rlen = $len / $self->{'nrows'};
+   my $idx = $rlen - 1;
+   for (my $r=0; $r<$len; $r+=$rlen) {
+      my $row = [ @{$ref}[$r ... $idx] ];
+      return $row if ($rlen eq $len);
+      push(@data, $row);
+      $idx += $rlen;
+   }
+   \@data;
+}
+
 # Check to make sure the values are numbers, greater than 0.
 # Throw error if any problems are found.
 sub _checkNumValidity {
@@ -334,8 +369,8 @@ sub _checkNumValidity {
       _error(8); 
    }
 
-# Very low cell counts (< 5) aren't great, either. Maybe a 
-# warning is in order.
+   # Very low cell counts (< 5) aren't great, either. Maybe a 
+   # warning is in order.
    if ($value < 5) {
       warn "Warning: Very small cell frequency (freq. < 5) found.\n",
 		"This will reduce the reliability of the test.\n";
@@ -373,6 +408,9 @@ sub _getHandle {
       if ($data =~ /\AGLOB/) {
          return _fileread($data);
       }
+      if ($data =~ /\AIO::File=GLOB/) {
+         return _fileread($data);
+      }
       else {
          _error(10);
       }
@@ -395,7 +433,7 @@ sub _getHandle {
 # Keep all the error strings in one place.
 sub _error {
    my ($code) = shift;
-   my ($package, $filename, $line) = caller(1);
+	my $errorbar = "\n!-----------------!\n";
    my @e = (
    "", # Normal execution
    "Input error: Must pass in a filename for a file containing the data.\n\tEx: \$g = new Statistics::Gtest(\$datafile);",
@@ -404,15 +442,18 @@ sub _error {
    "Input error: Data must be structured as array of array refs, with each array ref
    pointing to one row of data.",
    "Data inconsistency: No data found in table.",
-   "Data inconsistency: sum of table values do not match total.",
+   "Data inconsistency: sum of table values does not match total.",
    "Data inconsistency: could not read row or column.",
    "Data invalid: All data must be positive integers.",
    "Data invalid: Found a zero frequency value. Can't continue.",
    "Input error: Reference to invalid data type; must be reference to array.",
    );
 
-   warn " Error: in $package: file $filename, line $line\n\n";
-   warn " Error: ", $e[$code], "\n\n";
+	print $errorbar;
+	print "\n Error",Carp::shortmess();
+   print " ", $e[$code], "\n";
+	print Carp::longmess(" Stack trace:\n");
+	print $errorbar;
    exit $code; 
 }
 
@@ -438,29 +479,31 @@ Statistics::Gtest - calculate G-statistic for tabular data
     
 =head1 DESCRIPTION
 
-C<Statistics::Gtest> is a class that calculates the G-statistic for goodness of fit for
-frequency data. It can be used on simple frequency distributions (1-way
-tables) or for analyses of independence (2-way tables). 
+C<Statistics::Gtest> is a class that calculates the G-statistic for
+goodness of fit for frequency data. It can be used on simple frequency
+distributions (1-way tables) or for analyses of independence (2-way
+tables).
 
-Note that C<Statistics::Gtest> will B<not>, by itself, perform the significance test 
-for you -- it just provides the G-statistic that can then be compared with 
-the chi-square distribution to determine significance.
+Note that C<Statistics::Gtest> will B<not>, by itself, perform the
+significance test for you -- it just provides the G-statistic that
+can then be compared with the chi-square distribution to determine
+significance.
 
 =head1 OVERVIEW and EXAMPLES
 
 A goodness of fit test attempts to determine if an observed frequency
 distribution differs significantly from a hypothesized frequency
-distribution. From C<Statistics::Gtest>'s point of view, these tests come in two
-flavors: 1-way tests (where a single frequency distribution is tested 
-against an expected distribution) and 2-way tests (where a matrix of
-observed values is tested for independence -- that is, the lack of interaction
-effects among the two axes being measured). 
+distribution. From C<Statistics::Gtest>'s point of view, these tests
+come in two flavors: 1-way tests (where a single frequency distribution
+is tested against an expected distribution) and 2-way tests (where a
+matrix of observed values is tested for independence -- that is, the
+lack of interaction effects among the two axes being measured).
 
 A simple example might help here. You've grown 160 plants from seed
-produced by a single parent plant. You observe that among the offspring 
-plants, some have spiny leaves, some have hairy leaves, and some have smooth
-leaves. What is the likelihood that the distribution of this trait follows
-the expected values for simple Mendelian inheritance? 
+produced by a single parent plant. You observe that among the offspring
+plants, some have spiny leaves, some have hairy leaves, and some have
+smooth leaves. What is the likelihood that the distribution of this
+trait follows the expected values for simple Mendelian inheritance?
 
  Observed values:
    Spiny Hairy Smooth
@@ -469,21 +512,22 @@ the expected values for simple Mendelian inheritance?
  Expected values (for a 9:3:3:1 ratio):
      90    60    10
 
-If the observed and expected values are put into two files, C<Statistics::Gtest> can 
-create a G-statistic object that will calculate the likelihood that the
-observed distribution is significantly different from the distribution that
-would be expected by simple inheritance. (The value of G for this comparison
-is approximately 1.495, with 2 degrees of freedom; the observed results are
-not significantly different from expected at the .05 -- or even .1 level.)
+If the observed and expected values are put into two files,
+C<Statistics::Gtest> can create a G-statistic object that will calculate
+the likelihood that the observed distribution is significantly different
+from the distribution that would be expected by simple inheritance. (The
+value of G for this comparison is approximately 1.495, with 2 degrees
+of freedom; the observed results are not significantly different from
+expected at the .05 -- or even .1 level.)
 
 2-way tests will usually not need a table of expected values, as the
-expected values are generated from the observed value sums. However, one can
-be loaded for 2-way tables as well.
+expected values are generated from the observed value sums. However,
+one can be loaded for 2-way tables as well.
 
 To determine if the calculated G statistic indicates a statistically
-significant result, you will need to look up the values in
-a chi-square distribution on your own, or make use of the
-C<Statistics::Distributions> module: 
+significant result, you will need to look up the values in a chi-square
+distribution on your own, or make use of the C<Statistics::Distributions>
+module:
 
  use Statistics::Gtest;
  use Statistics::Distributions;
@@ -499,20 +543,31 @@ C<Statistics::Distributions> module:
    print "$g: Sig. at the $sv level. ($chis cutoff)\n"
  } 
 
-By default, C<Statistics::Gtest> returns a G statistic that has been modified by
-William's correction (Williams 1976). This correction reduces the value of G
-for smaller sample sizes, and has progressively less effect as the sample
-size increases. The raw, uncorrected G statistic is also available.
+By default, C<Statistics::Gtest> returns a G statistic that has been
+modified by William's correction (Williams 1976). This correction reduces
+the value of G for smaller sample sizes, and has progressively less
+effect as the sample size increases. The raw, uncorrected G statistic
+is also available.
 
-Calculation methods based on Sokal, R.R., and F.J. Rohlf, Biometry. 1981.
-W.H. Freeman and Company, San Francisco.
+=head3 References
 
-Williams, D.A. 1976. Improved likelihood ratio test for complete contingency
-tables. Biometrika, 63:33 - 37.
+=over 
 
-=head1 Public Methods
+=item 
 
-=head2 Constructor
+Sokal, R.R., and F.J. Rohlf, Biometry. 1981.  W.H. Freeman and Company,
+San Francisco.
+
+=item 
+
+Williams, D.A. 1976. Improved likelihood ratio test for complete
+contingency tables. Biometrika, 63:33 - 37.
+
+=back
+
+=head2 Public Methods
+
+=head3 Constructor
 
    $g = Statistics::Gtest->new($data);
    $g = new Statistics::Gtest($data);
@@ -528,14 +583,14 @@ Data in files must be arranged into rows and columns, separated by
 whitespace. In all cases,  must be B<no> non-numeric characters, B<no> 
 empty cells, and B<no> zero counts.  Arrays are B<not> valid input. 
 
-=head2 getG
+=head3 getG
 
    $float = $g->getG();
 
 Returns the corrected G-statistic for the current observed and expected
 frequency counts.
 
-=head2 getRawG
+=head3 getRawG
 
    $float = $g->getRawG();
 
@@ -543,26 +598,40 @@ Returns the uncorrected G-statistic for the current observed and expected
 frequency counts. This value can be misleadingly large for small sample
 sizes (n < 200).
 
-=head2 getQ
+=head3 getQ
 
    $float = $g->getQ();
 
 Returns Williams' correction (q) for this test.  (See explanation in
 'Overview and Examples'.)
 
-=head2 setExpected
+=head3 getObserved
+
+   $arrayref = $g->getObserved();
+
+Returns an array reference containing the observed cell values. The
+array is formatted in the same row-column layout as the input data.
+
+=head3 getExpected
+
+   $arrayref = $g->getExpected();
+
+Returns an array reference containing the expected cell values. The
+array is formatted in the same row-column layout as the observed data.
+
+=head3 setExpected
 
    $g->setExpected($string);
    $g->setExpected($arrayref);
    $g->setExpected($filename);
    $g->setExpected($filehandle);
 
-If testing with a specific hypothesized distribution, the expected frequency
-values for that distribution, given the total sample size, must be input
-to C<Statistics::Gtest>. The input data has the same contraints on format as does the
-initial data.  
+If testing with a specific hypothesized distribution, the expected
+frequency values for that distribution, given the total sample size,
+must be input to C<Statistics::Gtest>. The input data has the same
+contraints on format as does the initial data.
 
-=head2 getDF
+=head3 getDF
 
    $integer = $g->getDF();
 
@@ -570,53 +639,53 @@ Returns the current degrees of freedom for this distribution, which is
 calculated automatically from the observed data (I<rows> - 1 for 1-way
 tests, (I<rows> - 1) * (I<cols> - 1) for 2-way tests).
 
-=head2 setDF
+=head3 setDF
 
    $g->setDF($integer);
 
-Sets the degrees of freedom for this distribution. Sometimes this value
-needs to be modified beyond the standard rules used by C<Statistics::Gtest>; C<setDF>
-makes this possible. 
+Sets the degrees of freedom for this distribution. Sometimes
+this value needs to be modified beyond the standard rules used by
+C<Statistics::Gtest>; C<setDF> makes this possible.
 
-=head2 getRow
+=head3 getRow
 
    $rowref = $g->getRow(rownum);
 
 Returns a row from the array of observed data. Row numbering is
 zero-based.
 
-=head2 getCol
+=head3 getCol
 
    $colref = $g->getCol(colnum);
 
 Returns a column for the array of observed data. Column numbering is
 zero-based.
 
-=head2 rowSum
+=head3 rowSum
 
    $integer = $g->rowSum($index);
 
 Returns the sum of the requested row.
 
-=head2 colSum
+=head3 colSum
 
    $integer = $g->colSum($index);
 
 Returns the sum of the requested column.
 
-=head2 getRowNum
+=head3 getRowNum
 
    $integer = $g->getRowNum();
 
 Returns the number of rows in the data table.
 
-=head2 getColNum
+=head3 getColNum
 
    $integer = $g->getColNum();
 
 Returns the number of columns in the data table.
 
-=head2 getSumTotal
+=head3 getSumTotal
 
    $integer = $g->getSumTotal();
 
